/*
 * Copyright (C) 2019 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

syntax = "proto3";
package ignition.msgs;
option java_package = "com.ignition.msgs";
option java_outer_classname = "VideoRecordProtos";

/// \ingroup ignition.msgs
/// \interface VideoRecord
/// \brief A message that allows for control of video recording functions

import "ignition/msgs/header.proto";

message VideoRecord
{
  /// \brief Optional header data
  Header header           = 1;

  /// \brief True to start video recording
  bool start              = 2;

  /// \brief True to stop video recording
  bool stop               = 3;

  /// \brief Video encoding format, e.g. "mp4", "ogv"
  string format           = 4;

  /// \brief filename of the recorded video
  string save_filename    = 5;
}
