/*
 * Copyright (C) 2017 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

syntax = "proto3";
package ignition.msgs;
option java_package = "com.ignition.msgs";
option java_outer_classname = "AltimeterProtos";

/// \ingroup ignition.msgs
/// \interface Altimeter
/// \brief Data from an altimeter sensor

import "ignition/msgs/header.proto";

/// \brief Altimeter sensor data
message Altimeter
{
  // Other Optional header data
  Header header             = 1; /// Optional header data

  /// \brief Vertical position data, in meters.
  double vertical_position  = 2;

  /// \brief Vertical velocity data, in meters/second.
  double vertical_velocity  = 3;

  /// \brief Vertical reference.
  double vertical_reference = 4;
}
